@@ -89,9 +89,8 @@ class Connection(object):
     """Sends the given request to the dm_env_rpc server and returns the response.
 
     The request should be an instance of one of the dm_env_rpc Request messages,
-    such
-    as CreateWorldRequest.  Based on the type the correct payload for the
-    EnvironmentRequest will be constructed and set to the dm_env_rpc server.
+    such as CreateWorldRequest. Based on the type the correct payload for the
+    EnvironmentRequest will be constructed and sent to the dm_env_rpc server.
 
     Blocks until the server sends back its response.
 
@@ -101,22 +100,29 @@ class Connection(object):
 
     Returns:
       The response the dm_env_rpc server returned for the given RPC call,
-      unwrapped
-      from the EnvironmentStream message.  For instance if `request` had type
-      `CreateWorldRequest` this returns a message of type `CreateWorldResponse`.
+      unwrapped from the EnvironmentStream message.  For instance if `request`
+      had type `CreateWorldRequest` this returns a message of type
+      `CreateWorldResponse`.
 
     Raises:
       DmEnvRpcError: The dm_env_rpc server responded to the request with an
         error.
+      ValueError: The dm_env_rpc server responded to the request with an
+        unexpected response message.
     """
     field_name = self._type_to_field[type(request).__name__]
     environment_request = dm_env_rpc_pb2.EnvironmentRequest()
     getattr(environment_request, field_name).CopyFrom(request)
     self._stream.write(environment_request)
     response = self._stream.read()
-    if response.HasField('error'):
+    response_field = response.WhichOneof('payload')
+    if response_field == 'error':
       raise error.DmEnvRpcError(response.error)
-    return getattr(response, field_name)
+    elif response_field == field_name:
+      return getattr(response, field_name)
+    else:
+      raise ValueError(f'Unexpected response message! expected: {field_name}, '
+                       f'actual: {response_field}')
 
   def close(self):
     """Closes the connection.  Call when the connection is no longer needed."""
@@ -161,6 +167,9 @@ def create_secure_channel_and_connect(
       super().__init__(channel)
       self._channel = channel
 
+    def __del__(self):
+      self.close()
+
     def close(self):
       super().close()
       self._channel.close()
