@@ -14,7 +14,6 @@
 # ============================================================================
 """Tests for Connection."""
 
-from concurrent import futures
 import contextlib
 
 from absl.testing import absltest
@@ -29,8 +28,6 @@ from dm_env_rpc.v1 import dm_env_rpc_pb2
 from dm_env_rpc.v1 import error
 from dm_env_rpc.v1 import tensor_utils
 
-_SUEZ_SERVICE = dm_env_rpc_pb2.DESCRIPTOR.services_by_name['Environment']
-
 _CREATE_REQUEST = dm_env_rpc_pb2.CreateWorldRequest(
     settings={'foo': tensor_utils.pack_tensor('bar')})
 _CREATE_RESPONSE = dm_env_rpc_pb2.CreateWorldResponse()
@@ -39,6 +36,11 @@ _BAD_CREATE_REQUEST = dm_env_rpc_pb2.CreateWorldRequest()
 _TEST_ERROR = dm_env_rpc_pb2.EnvironmentResponse(
     error=status_pb2.Status(message='A test error.'))
 
+_INCORRECT_RESPONSE_TEST_MSG = dm_env_rpc_pb2.DestroyWorldRequest(
+    world_name='foo')
+_INCORRECT_RESPONSE = dm_env_rpc_pb2.EnvironmentResponse(
+    leave_world=dm_env_rpc_pb2.LeaveWorldResponse())
+
 _EXTENSION_REQUEST = struct_pb2.Value(string_value='extension request')
 _EXTENSION_RESPONSE = struct_pb2.Value(number_value=555)
 
@@ -60,6 +62,9 @@ _REQUEST_RESPONSE_PAIRS = {
         extension=_wrap_in_any(_EXTENSION_REQUEST)).SerializeToString():
         dm_env_rpc_pb2.EnvironmentResponse(
             extension=_wrap_in_any(_EXTENSION_RESPONSE)),
+    dm_env_rpc_pb2.EnvironmentRequest(
+        destroy_world=_INCORRECT_RESPONSE_TEST_MSG).SerializeToString():
+        _INCORRECT_RESPONSE,
 }
 
 
@@ -103,21 +108,47 @@ class ConnectionTests(absltest.TestCase):
         expected_response.Pack(_EXTENSION_RESPONSE)
         self.assertEqual(expected_response, response)
 
-  def test_create_secure_channel_and_connect(self):
-    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
-    port = server.add_secure_port('[::]:0', grpc.local_server_credentials())
-    server.start()
+  @mock.patch.object(grpc, 'secure_channel')
+  @mock.patch.object(grpc, 'channel_ready_future')
+  def test_create_secure_channel_and_connect(self, mock_channel_ready,
+                                             mock_secure_channel):
+    mock_channel = mock.MagicMock()
+    mock_secure_channel.return_value = mock_channel
 
     self.assertIsNotNone(
         dm_env_rpc_connection.create_secure_channel_and_connect(
-            f'[::]:{port}', grpc.local_channel_credentials()))
-    server.stop(grace=None)
+            'valid_address', grpc.local_channel_credentials()))
+
+    mock_channel_ready.assert_called_once_with(mock_channel)
+    mock_secure_channel.assert_called_once()
+    mock_channel.close.assert_called_once()
+
+  @mock.patch.object(grpc, 'secure_channel')
+  @mock.patch.object(grpc, 'channel_ready_future')
+  def test_create_secure_channel_and_connect_context(self, mock_channel_ready,
+                                                     mock_secure_channel):
+    mock_channel = mock.MagicMock()
+    mock_secure_channel.return_value = mock_channel
+
+    with dm_env_rpc_connection.create_secure_channel_and_connect(
+        'valid_address') as connection:
+      self.assertIsNotNone(connection)
+
+    mock_channel_ready.assert_called_once_with(mock_channel)
+    mock_secure_channel.assert_called_once()
+    mock_channel.close.assert_called_once()
 
   def test_create_secure_channel_and_connect_timeout(self):
     with self.assertRaises(grpc.FutureTimeoutError):
       dm_env_rpc_connection.create_secure_channel_and_connect(
           'invalid_address', grpc.local_channel_credentials(), timeout=1.)
 
+  def test_incorrect_response(self):
+    with _create_mock_channel() as mock_channel:
+      with dm_env_rpc_connection.Connection(mock_channel) as connection:
+        with self.assertRaisesRegex(ValueError, 'Unexpected response message'):
+          connection.send(_INCORRECT_RESPONSE_TEST_MSG)
+
 
 if __name__ == '__main__':
   absltest.main()
