@@ -201,6 +201,21 @@ class DmEnvAdaptorTests(absltest.TestCase):
     self.assertIsNone(timestep.discount)
     self.assertEqual({'foo': 5, 'bar': 'goodbye'}, timestep.observation)
 
+  def test_spec_generate_value_step(self):
+    self._connection.send = mock.MagicMock(return_value=_SAMPLE_STEP_RESPONSE)
+    action_spec = self._env.action_spec()
+    actions = {
+        name: spec.generate_value() for name, spec in action_spec.items()
+    }
+    self._env.step(actions)
+    self._connection.send.assert_called_once_with(
+        dm_env_rpc_pb2.StepRequest(
+            requested_observations=[1, 2],
+            actions={
+                1: tensor_utils.pack_tensor(actions['foo']),
+                2: tensor_utils.pack_tensor(actions['bar'], dtype=np.str_)
+            }))
+
   def test_reset_changes_spec_raises_error(self):
     self._connection.send = mock.MagicMock(return_value=_SAMPLE_STEP_RESPONSE)
     self._env.step({'foo': 4, 'bar': 'hello'})
@@ -213,14 +228,14 @@ class DmEnvAdaptorTests(absltest.TestCase):
   def test_observation_spec(self):
     expected_spec = {
         'foo': specs.Array(shape=(), dtype=np.uint8, name='foo'),
-        'bar': specs.Array(shape=(), dtype=np.str_, name='bar')
+        'bar': specs.StringArray(shape=(), name='bar')
     }
     self.assertEqual(expected_spec, self._env.observation_spec())
 
   def test_action_spec(self):
     expected_spec = {
         'foo': specs.Array(shape=(), dtype=np.uint8, name='foo'),
-        'bar': specs.Array(shape=(), dtype=np.str_, name='bar')
+        'bar': specs.StringArray(shape=(), name='bar')
     }
     self.assertEqual(expected_spec, self._env.action_spec())
 
@@ -310,8 +325,7 @@ class ReservedKeywordTests(absltest.TestCase):
         specs.Array(shape=(), dtype=np.uint8), self._env.reward_spec())
 
   def test_discount_spec(self):
-    self.assertEqual(
-        specs.Array(shape=(), dtype=np.str_), self._env.discount_spec())
+    self.assertEqual(specs.StringArray(shape=()), self._env.discount_spec())
 
   def test_reward_from_reserved_keyword(self):
     self._connection.send = mock.MagicMock(return_value=_RESERVED_STEP_RESPONSE)
@@ -341,8 +355,7 @@ class EnvironmentAutomaticallyRequestsReservedKeywords(absltest.TestCase):
         specs.Array(shape=(), dtype=np.uint8), self._env.reward_spec())
 
   def test_discount_spec_unrequested(self):
-    self.assertEqual(
-        specs.Array(shape=(), dtype=np.str_), self._env.discount_spec())
+    self.assertEqual(specs.StringArray(shape=()), self._env.discount_spec())
 
   def test_does_not_give_back_unrequested_observations(self):
     timestep = self._env.step({})
@@ -481,6 +494,21 @@ class EnvironmentNestedActionsObservations(absltest.TestCase):
 
 class CreateJoinHelpers(absltest.TestCase):
 
+  def test_create_world(self):
+    connection = mock.MagicMock()
+    connection.send = mock.MagicMock(
+        return_value=dm_env_rpc_pb2.CreateWorldResponse(
+            world_name='Damogran_01'))
+
+    world_name = dm_env_adaptor.create_world(connection, {'planet': 'Damogran'})
+    self.assertEqual('Damogran_01', world_name)
+
+    connection.send.assert_called_once_with(
+        text_format.Parse(
+            """settings: {
+                key: 'planet', value: { strings: { array: 'Damogran' } }
+            }""", dm_env_rpc_pb2.CreateWorldRequest()))
+
   def test_join_world(self):
     connection = mock.MagicMock()
     connection.send = mock.MagicMock(
@@ -586,19 +614,64 @@ class CreateJoinHelpers(absltest.TestCase):
         extensions={'extension': _ExampleExtension()})
     self.assertEqual('bar', env.extension.foo())
 
+  def test_create_join_world_with_invalid_extension(self):
+    connection = mock.MagicMock()
+    connection.send = mock.MagicMock(side_effect=[
+        dm_env_rpc_pb2.CreateWorldResponse(world_name='foo'),
+        dm_env_rpc_pb2.JoinWorldResponse(specs=_SAMPLE_SPEC),
+        dm_env_rpc_pb2.LeaveWorldResponse(),
+        dm_env_rpc_pb2.DestroyWorldRequest()
+    ])
+
+    with self.assertRaisesRegex(ValueError,
+                                'DmEnvAdaptor already has attribute'):
+      _ = dm_env_adaptor.create_and_join_world(
+          connection,
+          create_world_settings={},
+          join_world_settings={},
+          extensions={'step': object()})
+
+    connection.send.assert_has_calls([
+        mock.call(dm_env_rpc_pb2.CreateWorldRequest()),
+        mock.call(dm_env_rpc_pb2.JoinWorldRequest(world_name='foo')),
+        mock.call(dm_env_rpc_pb2.LeaveWorldRequest()),
+        mock.call(dm_env_rpc_pb2.DestroyWorldRequest(world_name='foo'))
+    ])
+
   def test_created_but_failed_to_join_world(self):
     connection = mock.MagicMock()
     connection.send = mock.MagicMock(
         side_effect=(
             dm_env_rpc_pb2.CreateWorldResponse(world_name='Damogran_01'),
             error.DmEnvRpcError(status_pb2.Status(message='Failed to Join.')),
-            dm_env_rpc_pb2.LeaveWorldResponse(),
             dm_env_rpc_pb2.DestroyWorldResponse()))
 
     with self.assertRaisesRegex(error.DmEnvRpcError, 'Failed to Join'):
       _ = dm_env_adaptor.create_and_join_world(
           connection, create_world_settings={}, join_world_settings={})
 
+    connection.send.assert_has_calls([
+        mock.call(dm_env_rpc_pb2.CreateWorldRequest()),
+        mock.call(dm_env_rpc_pb2.JoinWorldRequest(world_name='Damogran_01')),
+        mock.call(dm_env_rpc_pb2.DestroyWorldRequest(world_name='Damogran_01'))
+    ])
+
+  def test_created_and_joined_but_adaptor_failed(self):
+    connection = mock.MagicMock()
+    connection.send = mock.MagicMock(
+        side_effect=(
+            dm_env_rpc_pb2.CreateWorldResponse(world_name='Damogran_01'),
+            dm_env_rpc_pb2.JoinWorldResponse(specs=_SAMPLE_SPEC),
+            dm_env_rpc_pb2.LeaveWorldResponse(),
+            dm_env_rpc_pb2.DestroyWorldResponse()))
+
+    with self.assertRaisesRegex(ValueError, 'Unsupported observations'):
+      _ = dm_env_adaptor.create_and_join_world(
+          connection,
+          create_world_settings={},
+          join_world_settings={},
+          requested_observations=['invalid_observation'])
+
     connection.send.assert_has_calls([
         mock.call(dm_env_rpc_pb2.CreateWorldRequest()),
         mock.call(dm_env_rpc_pb2.JoinWorldRequest(world_name='Damogran_01')),
