@@ -31,14 +31,14 @@ _SCALAR_VALUE_TYPES = frozenset(
     ('float', 'double', 'int8', 'int32', 'int64', 'uint8', 'uint32', 'uint64'))
 
 
-def _can_cast(array_or_scalar, np_dtype):
+def _can_cast(array_or_scalar, np_dtype: np.dtype) -> bool:
   for value in np.asarray(array_or_scalar).flat:
     if not np.can_cast(value, np_dtype, casting='safe'):
       return False
   return True
 
 
-def _np_range_info(np_type):
+def _np_range_info(np_type: np.dtype):
   """Returns type info for `np_type`, which includes min and max attributes."""
   if issubclass(np_type, np.floating):
     return np.finfo(np_type)
@@ -80,7 +80,7 @@ def _get_value(min_max_value, shape, default):
   return min_max
 
 
-def bounds(tensor_spec):
+def bounds(tensor_spec: dm_env_rpc_pb2.TensorSpec) -> Bounds:
   """Gets the inclusive bounds of `tensor_spec`.
 
   Args:
@@ -154,20 +154,6 @@ def set_bounds(tensor_spec: dm_env_rpc_pb2.TensorSpec, minimum, maximum):
   has_min = minimum is not None
   has_max = maximum is not None
 
-  if has_min:
-    minimum = np.asarray(minimum)
-    if minimum.size != 1 and minimum.shape != tuple(tensor_spec.shape):
-      raise ValueError(
-          f'minimum has shape {minimum.shape}, which is incompatible with '
-          f"tensor_spec {tensor_spec.name}'s shape {tensor_spec.shape}.")
-
-  if has_max:
-    maximum = np.asarray(maximum)
-    if maximum.size != 1 and maximum.shape != tuple(tensor_spec.shape):
-      raise ValueError(
-          f'maximum has shape {maximum.shape}, which is incompatible with '
-          f"tensor_spec {tensor_spec.name}'s shape {tensor_spec.shape}.")
-
   if ((has_min and not _can_cast(minimum, np_type)) or
       (has_max and not _can_cast(maximum, np_type))):
     raise ValueError(
@@ -177,6 +163,20 @@ def set_bounds(tensor_spec: dm_env_rpc_pb2.TensorSpec, minimum, maximum):
             maximum=maximum,
             dtype=dm_env_rpc_pb2.DataType.Name(tensor_spec.dtype)))
 
+  if has_min:
+    minimum = np.asarray(minimum, dtype=np_type)
+    if minimum.size != 1 and minimum.shape != tuple(tensor_spec.shape):
+      raise ValueError(
+          f'minimum has shape {minimum.shape}, which is incompatible with '
+          f"tensor_spec {tensor_spec.name}'s shape {tensor_spec.shape}.")
+
+  if has_max:
+    maximum = np.asarray(maximum, dtype=np_type)
+    if maximum.size != 1 and maximum.shape != tuple(tensor_spec.shape):
+      raise ValueError(
+          f'maximum has shape {maximum.shape}, which is incompatible with '
+          f"tensor_spec {tensor_spec.name}'s shape {tensor_spec.shape}.")
+
   if (has_min and has_max and np.any(maximum < minimum)):
     raise ValueError('TensorSpec "{}" has min {} larger than max {}.'.format(
         tensor_spec.name, minimum, maximum))
