@@ -21,6 +21,8 @@ from absl.testing import parameterized
 import mock
 import numpy as np
 
+from google.protobuf import any_pb2
+from google.protobuf import struct_pb2
 from dm_env_rpc.v1 import dm_env_rpc_pb2
 from dm_env_rpc.v1 import tensor_utils
 
@@ -54,6 +56,26 @@ class PackTensorTests(parameterized.TestCase):
     actual = struct.unpack(fmt, getattr(tensor, expected_payload).array)
     self.assertEqual(scalar, actual)
 
+  def test_pack_scalar_protos(self):
+    scalar = struct_pb2.Value(string_value='my message')
+    tensor = tensor_utils.pack_tensor(scalar)
+    self.assertEqual([], tensor.shape)
+    self.assertLen(tensor.protos.array, 1)
+    unpacked = struct_pb2.Value()
+    self.assertTrue(tensor.protos.array[0].Unpack(unpacked))
+    self.assertEqual(scalar, unpacked)
+
+  def test_pack_scalar_any_proto(self):
+    scalar = struct_pb2.Value(string_value='my message')
+    scalar_any = any_pb2.Any()
+    scalar_any.Pack(scalar)
+    tensor = tensor_utils.pack_tensor(scalar_any)
+    self.assertEqual([], tensor.shape)
+    self.assertLen(tensor.protos.array, 1)
+    unpacked = struct_pb2.Value()
+    self.assertTrue(tensor.protos.array[0].Unpack(unpacked))
+    self.assertEqual(scalar, unpacked)
+
   @parameterized.parameters(
       (25, np.float32, 'floats'),
       (25, np.float64, 'doubles'),
@@ -102,6 +124,23 @@ class PackTensorTests(parameterized.TestCase):
     packed_array = getattr(tensor, expected_payload).array
     np.testing.assert_array_equal(array, packed_array)
 
+  def test_pack_proto_arrays(self):
+    array = np.array([
+        struct_pb2.Value(string_value=message)
+        for message in ['foo', 'bar']
+    ])
+    tensor = tensor_utils.pack_tensor(array)
+    self.assertEqual([2], tensor.shape)
+    unpacked = struct_pb2.Value()
+    tensor.protos.array[0].Unpack(unpacked)
+    self.assertEqual(array[0], unpacked)
+    tensor.protos.array[1].Unpack(unpacked)
+    self.assertEqual(array[1], unpacked)
+
+  def test_pack_mixed_proto_array_fails(self):
+    with self.assertRaisesRegex(ValueError, 'not recognized'):
+      tensor_utils.pack_tensor(np.array([struct_pb2.Value(), 1, 2, 3]))
+
   def test_packed_rowmajor(self):
     array2d = np.array([[1, 2], [3, 4], [5, 6]], dtype=np.int32)
     tensor = tensor_utils.pack_tensor(array2d)
@@ -116,6 +155,31 @@ class PackTensorTests(parameterized.TestCase):
     with self.assertRaises(ValueError):
       tensor_utils.pack_tensor([[1, 2], [3, 4, 5]])
 
+  @parameterized.parameters(
+      (['foo', 'bar'], np.str_),
+      ('baz', dm_env_rpc_pb2.DataType.STRING),
+      (['foobar'], np.array(['foobar']).dtype),
+  )
+  def test_np_object_strings(self, value, dtype):
+    object_array = np.array(value, dtype=np.object)
+    tensor = tensor_utils.pack_tensor(object_array, dtype=dtype)
+    self.assertEqual(list(object_array.shape), tensor.shape)
+    self.assertTrue(tensor.HasField('strings'))
+
+  def test_np_object_strings_no_dtype_raises_exception(self):
+    with self.assertRaises(ValueError):
+      tensor_utils.pack_tensor(np.array(['foo'], dtype=np.object))
+
+  @parameterized.parameters(
+      (['foo', 42, 'bar'],),
+      ([1, 2, 3],),
+  )
+  def test_np_object_to_strings_fail(self, bad_element):
+    with self.assertRaisesRegex(TypeError,
+                                'not all elements are Python string types'):
+      tensor_utils.pack_tensor(
+          np.array(bad_element, dtype=np.object), dtype=np.str_)
+
   def test_class_instance_throw_exception(self):
 
     class Foo(object):
@@ -181,6 +245,14 @@ class UnpackTensorTests(parameterized.TestCase):
     round_trip = tensor_utils.unpack_tensor(tensor)
     self.assertEqual(scalar, round_trip)
 
+  def test_unpack_scalar_proto(self):
+    scalar = struct_pb2.Value(string_value='my message')
+    tensor = tensor_utils.pack_tensor(scalar)
+
+    unpacked = struct_pb2.Value()
+    tensor_utils.unpack_tensor(tensor).Unpack(unpacked)
+    self.assertEqual(scalar, unpacked)
+
   @parameterized.parameters(
       ([np.float32(2.5), np.float32(3.5)],),
       ([np.float64(2.5), np.float64(3.5)],),
@@ -198,6 +270,20 @@ class UnpackTensorTests(parameterized.TestCase):
     round_trip = tensor_utils.unpack_tensor(tensor)
     np.testing.assert_array_equal(array, round_trip)
 
+  def test_unpack_proto_arrays(self):
+    array = np.array([
+        struct_pb2.Value(string_value=message)
+        for message in ['foo', 'bar']
+    ])
+    tensor = tensor_utils.pack_tensor(array)
+    round_trip = tensor_utils.unpack_tensor(tensor)
+
+    unpacked = struct_pb2.Value()
+    round_trip[0].Unpack(unpacked)
+    self.assertEqual(array[0], unpacked)
+    round_trip[1].Unpack(unpacked)
+    self.assertEqual(array[1], unpacked)
+
   def test_unpack_multidimensional_arrays(self):
     tensor = dm_env_rpc_pb2.Tensor()
     tensor.floats.array[:] = [1, 2, 3, 4, 5, 6, 7, 8]
@@ -324,9 +410,9 @@ class DataTypeToNpTypeTests(absltest.TestCase):
         np.float32,
         tensor_utils.data_type_to_np_type(dm_env_rpc_pb2.DataType.FLOAT))
 
-  def test_proto_type(self):
-    with self.assertRaises(TypeError):
-      tensor_utils.data_type_to_np_type(dm_env_rpc_pb2.DataType.PROTO)
+  def test_empty_object_list(self):
+    tensor = tensor_utils.pack_tensor(np.array([], dtype=np.object))
+    self.assertEqual([0], tensor.shape)
 
   def test_unknown_type(self):
     with self.assertRaises(TypeError):
@@ -364,7 +450,7 @@ class GetPackerTests(absltest.TestCase):
   def test_can_pack(self):
     packer = tensor_utils.get_packer(np.int32)
     tensor = dm_env_rpc_pb2.Tensor()
-    packer.pack(tensor, np.asarray([1, 2, 3]))
+    packer.pack(tensor, np.asarray([1, 2, 3], dtype=np.int32))
     self.assertEqual([1, 2, 3], tensor.int32s.array)
 
   def test_can_unpack(self):
